/* 
   Unix SMB/CIFS implementation.

   AIX loadable authentication module, providing identification 
   routines against Samba winbind/Windows NT Domain

   Copyright (C) Aaron Collins 2003
   
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.   
*/

#include "winbind_client.h"
#include <nsswitch.h>
#include <iaf.h>
#include <ia.h>


#define NSDB_GROUP		"group"
#define NSDB_PASSWD		"passwd"

#define NSS_METHOD_PROTOTYPE(method) \
	int method(void *, void *, va_list)

/* Make sure that the module gets registered needed by freebsd 5.1 */

extern enum nss_status _nss_winbind_getgrent_r(struct group *, char *, size_t,
    int *);
extern enum nss_status _nss_winbind_getgrnam_r(const char *, struct group *,
    char *, size_t, int *);
extern enum nss_status _nss_winbind_getgrgid_r(gid_t gid, struct group *, char *,
    size_t, int *);
extern enum nss_status _nss_winbind_setgrent(void);
extern enum nss_status _nss_winbind_endgrent(void);

extern enum nss_status _nss_winbind_getpwent_r(struct passwd *, char *, size_t,
    int *);
extern enum nss_status _nss_winbind_getpwnam_r(const char *, struct passwd *,
    char *, size_t, int *);
extern enum nss_status _nss_winbind_getpwuid_r(gid_t gid, struct passwd *, char *,
    size_t, int *);
extern enum nss_status _nss_winbind_setpwent(void);
extern enum nss_status _nss_winbind_endpwent(void);

NSS_METHOD_PROTOTYPE(__nss_compat_getgrnam_r);
NSS_METHOD_PROTOTYPE(__nss_compat_getgrgid_r);
NSS_METHOD_PROTOTYPE(__nss_compat_getgrent_r);
NSS_METHOD_PROTOTYPE(__nss_compat_setgrent);
NSS_METHOD_PROTOTYPE(__nss_compat_endgrent);

NSS_METHOD_PROTOTYPE(__nss_compat_getpwnam_r);
NSS_METHOD_PROTOTYPE(__nss_compat_getpwuid_r);
NSS_METHOD_PROTOTYPE(__nss_compat_getpwent_r);
NSS_METHOD_PROTOTYPE(__nss_compat_setpwent);
NSS_METHOD_PROTOTYPE(__nss_compat_endpwent);

NSS_METHOD_PROTOTYPE(_nss_winbind_openinfo);

static ns_mtab methods[] = {
{ NSDB_GROUP, "getgrnam_r", __nss_compat_getgrnam_r, (void *) _nss_winbind_getgrnam_r },
{ NSDB_GROUP, "getgrgid_r", __nss_compat_getgrgid_r, _nss_winbind_getgrgid_r },
{ NSDB_GROUP, "getgrent_r", __nss_compat_getgrent_r, _nss_winbind_getgrent_r },
{ NSDB_GROUP, "endgrent",   __nss_compat_setgrent,   _nss_winbind_setgrent },
{ NSDB_GROUP, "setgrent",   __nss_compat_endgrent,   _nss_winbind_endgrent },

{ NSDB_PASSWD, "getpwnam_r", __nss_compat_getpwnam_r, _nss_winbind_getpwnam_r },
{ NSDB_PASSWD, "getpwuid_r", __nss_compat_getpwuid_r, _nss_winbind_getpwuid_r },
{ NSDB_PASSWD, "getpwent_r", __nss_compat_getpwent_r, _nss_winbind_getpwent_r },
{ NSDB_PASSWD, "endpwent",   __nss_compat_setpwent,   _nss_winbind_setpwent },
{ NSDB_PASSWD, "setpwent",   __nss_compat_endpwent,   _nss_winbind_endpwent },

{ "iaf",	"ia_openinfo",	_nss_winbind_openinfo, 0},
};

ns_mtab *
nss_module_register(const char *source, unsigned int *mtabsize,
    nss_module_unregister_fn *unreg)
{
        *mtabsize = sizeof(methods)/sizeof(methods[0]);
        *unreg = NULL;
        return (methods);
}

/* int ia_openinfo(const char  * logname, uinfo_t  * uinfo); */
int 
_nss_winbind_openinfo(void *rv, void *mdata, va_list ap)
{
	const char *name;
	struct passwd p;
	char buf[512];
	int *errnop, *ret = rv;
	enum nss_status status;
	int n, dlen, slen;
	uinfo_t *uipp = NULL;
	uinfo_t uip;

	name = va_arg(ap, const char *);
	uipp = va_arg(ap, uinfo_t *);

	*uipp = 0;
	
	status = _nss_winbind_getpwnam_r(name, &p, buf, 512, errnop);
	status = __nss_compat_result(status, *errnop);

	if (status == NS_SUCCESS) {
		dlen = strlen(p.pw_dir);
		slen = strlen(p.pw_shell);

		/* struct + 1 group + dir + shell */
		n = sizeof(*uip) + 1 + sizeof(gid_t) + dlen + 1 + slen + 1;

		if ((uip = malloc(n)) == NULL) {
			return NS_TRYAGAIN;
		}

		bzero(uip, n);

		strncpy(uip->ia_name, p.pw_name, sizeof(uip->ia_name));
		strncpy(uip->ia_pwdp, p.pw_passwd, sizeof(uip->ia_pwdp));
		uip->ia_uid = p.pw_uid;
		uip->ia_gid = p.pw_gid;
		uip->ia_lstchg = -1;
		uip->ia_min = -1;
		uip->ia_max = -1;
		uip->ia_warn = -1;
		uip->ia_inact = -1;
		uip->ia_expire = -1;

		uip->ia_dirsz = dlen;
		uip->ia_shsz = slen;
		uip->ia_lvlcnt = 0;
		uip->ia_sgidcnt = 1;

		uip->ia_lvlp = (level_t *)&uip[1];
		uip->ia_sgidp = (gid_t *)&uip[1];
		uip->ia_dirp = (char *)&uip->ia_sgidp[1];
		uip->ia_shellp = &uip->ia_dirp[dlen + 1];
		*uip->ia_sgidp = p.pw_gid;
		strcpy(uip->ia_dirp, p.pw_dir);
		strcpy(uip->ia_shellp, p.pw_shell);

		*ret = 0;
		*uipp = uip;
	}

	fprintf(stderr, "ia_openinfo: status=%d\n", status);
	return status;
}
